//+------------------------------------------------------------------+
//| 10:43 AM 10/1/2014                                     chart.mqh |
//|                               Copyright  2014, William H Roeder |
//|                                        mailto:WHRoeder@yahoo_com |
//| Base class of all chart types (time, price and tick.)            |
//+------------------------------------------------------------------+
#ifndef CHART_MQH
#define CHART_MQH
class cChart{
 public:
   double                Open(int iBar=0)          const{return  mOpen[iBar];  }
   double                High(int iBar=0)          const{return  mHigh[iBar];  }
   double                 Low(int iBar=0)          const{return   mLow[iBar];  }
   double               Close(int iBar=0)          const{return mClose[iBar];  }
   int                  Bars(void)                const{return ArraySize(mLow);}
   int                  iHighest(    int n = WHOLE_ARRAY, int iBeg = 0) const;
   int                   iLowest(    int n = WHOLE_ARRAY, int iBeg = 0) const;
   int                  iHighestHigh(int n = WHOLE_ARRAY, int iBeg = 0) const;
   int                   iLowestLow( int n = WHOLE_ARRAY, int iBeg = 0) const;
 protected:
                        cChart(void){}
   void                 Initialize(double p, int nBars = 1);
   void                 Insert(double p)        {Initialize(p, this.Bars()+1); }
   void                 SetHigh(double p)               { mHigh[0]   = p;      }
   void                 SetLow(double p)                {  mLow[0]   = p;      }
   void                 SetClose(double p)              {mClose[0]   = p;      }
 private:
   void                 Enlarge(double& A[], int nBars, double p);

   double                mOpen[];
   double                mHigh[];
   double                 mLow[];
   double               mClose[];
}; // cChart
//+- cChart ------------------------------------------------ public -+
int                  cChart::iHighest(int n = WHOLE_ARRAY, int iBeg = 0) const{
   int nBars = Bars();
   if(n == WHOLE_ARRAY || n + iBeg > nBars)  n = nBars - iBeg;
   return ArrayMaximum(mHigh, n, iBeg);   // Lowest index with maximum.
}
int                  cChart::iLowest(int n = WHOLE_ARRAY, int iBeg = 0) const{
   int nBars = Bars();
   if(n == WHOLE_ARRAY || n + iBeg > nBars)  n = nBars - iBeg;
   return ArrayMinimum(mLow, n, iBeg);
}
int               cChart::iHighestHigh(int n = WHOLE_ARRAY, int iBeg = 0) const{
   int      nBars    = Bars();   if(n == WHOLE_ARRAY) n = nBars;
   int      iEnd     = MathMin(nBars, iBeg + n);
   int      iLE      = iBeg;
   for(double  LE = mHigh[iBeg]; iBeg < iEnd; ++iBeg){
      double   value = mHigh[iBeg];
      if(value >= LE){
         iLE   = iBeg;     LE = value;
         iEnd  = MathMin(nBars, iBeg + n);
   }  }
   return iLE; // Highest index with maximum
}
int               cChart::iLowestLow(int n = WHOLE_ARRAY, int iBeg = 0) const{
   int      nBars    = Bars();   if(n == WHOLE_ARRAY) n = nBars;
   int      iEnd     = MathMin(nBars, iBeg + n);
   int      iLE      = iBeg;
   for(double  LE = mLow[iBeg];  iBeg < iEnd; ++iBeg){
      double   value = mLow[iBeg];
      if(value <= LE){
         iLE   = iBeg;     LE = value;
         iEnd  = MathMin(nBars, iBeg + n);
   }  }
   return iLE; // Lowest index with minimum
}
//+- cChart --------------------------------------------- protected -+
void                 cChart::Initialize(double p, int nBars = 1){
   Enlarge( mOpen, nBars, p);    Enlarge(mHigh, nBars, p);
   Enlarge(mClose, nBars, p);    Enlarge( mLow, nBars, p);
}
//+- cChart ----------------------------------------------- private -+
void                 cChart::Enlarge(double& A[], int nBars, double p){
   bool isSeries = ArrayGetAsSeries(A);
   ArraySetAsSeries(A, !isSeries);
      ArrayResize(A, nBars);  // Shift values A[2]=A[1]; A[1]=A[0]; A[0]=p
   ArraySetAsSeries(A,  isSeries);
   A[0] = p;
}
//+------------------------------------------------------------------+
#endif   // CHART_MQH
